unit Game;

interface

	uses
		QDOffscreen, Globals, Dialogs;

{----------------------------}

	procedure SetupGameGlobals;
	procedure SetNewPosition (who: Names);
	procedure ControlsHandler;
	procedure PelletCollisionHandler;
	function WallCollisionTest (characterRect: Rect): boolean;

{----------------------------}

implementation

{----------------------------}

	procedure SetupGameGlobals;

	begin
		case gameMenus.windowSelection of
			Small: 
				begin
					gameGlobals.ticksPerFrame := 4;
					gameGlobals.borderLeft := 128;
					gameGlobals.borderTop := 8;
					gameGlobals.borderRight := 369;
					gameGlobals.borderBottom := 278;
{         gameGlobals.incrementSize := 3;6 sound clicks}
					gameGlobals.incrementSize := 5;
					gameGlobals.pelletsConsumed := 0;
					pacmanInfo.direction := Left;
					pacmanInfo.frameNo := 0;
					pacmanInfo.positionLeft := 248;
					pacmanInfo.positionTop := 220;
				end;

			Medium: 
				begin
					gameGlobals.ticksPerFrame := 3;
					gameGlobals.borderLeft := 131;
					gameGlobals.borderTop := 10;
					gameGlobals.borderRight := 482;
					gameGlobals.borderBottom := 403;
{         gameGlobals.incrementSize := 5;7 sound clicks}
					gameGlobals.incrementSize := 7;
					gameGlobals.pelletsConsumed := 0;
					pacmanInfo.direction := Left;
					pacmanInfo.frameNo := 0;
					pacmanInfo.positionLeft := 308;
					pacmanInfo.positionTop := 319;
				end;

			Large: 
				begin
					gameGlobals.ticksPerFrame := 2;
					gameGlobals.borderLeft := 200;
					gameGlobals.borderTop := 15;
					gameGlobals.borderRight := 780;
					gameGlobals.borderBottom := 665;
{         gameGlobals.incrementSize := 10;12 sound clicks}
					gameGlobals.incrementSize := 11;
					gameGlobals.pelletsConsumed := 0;
					pacmanInfo.direction := Left;
					pacmanInfo.frameNo := 0;
					pacmanInfo.positionLeft := 492;
					pacmanInfo.positionTop := 526;
				end;

			otherwise
				AlertWithMessage('SetupGameGlobals case gameMenus.windowSelection error, invalid case');
		end;
	end;

{----------------------------}

	procedure SetNewPosition (who: Names);

		var
			wallCollisionFlag: boolean;
			nextLeft, nextTop, insetPadding: integer;
			nextRect: Rect;

	begin
		case gameMenus.windowSelection of
			Small: 
				insetPadding := 1;
			Medium: 
				insetPadding := 1;
			Large: 
				insetPadding := 4;

			otherwise
				AlertWithMessage('SetNewPosition case gameMenus.windowSelection error, invalid case');
		end;

		case who of
			PacMan: 
				begin
					with pacmanInfo do
						begin
							case direction of

								Left: 
									begin
										incrementX := gameGlobals.incrementSize * (-1);
										incrementY := 0;
										nextLeft := positionLeft + incrementX;
										nextTop := positionTop + incrementY;
										SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
										InsetRect(nextRect, insetPadding, insetPadding);

										wallCollisionFlag := WallCollisionTest(nextRect);
										if wallCollisionFlag then
											direction := Stay
										else
											begin
												positionLeft := nextLeft;
												positionTop := nextTop;
											end;

										if positionLeft < gameGlobals.borderLeft then
											begin
												positionLeft := gameGlobals.borderLeft;
												direction := Stay;
												frameNo := 0;
											end;
									end;

								Up: 
									begin
										incrementX := 0;
										incrementY := gameGlobals.incrementSize * (-1);
										nextLeft := positionLeft + incrementX;
										nextTop := positionTop + incrementY;
										SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
										InsetRect(nextRect, insetPadding, insetPadding);

										wallCollisionFlag := WallCollisionTest(nextRect);
										if wallCollisionFlag then
											direction := Stay
										else
											begin
												positionLeft := nextLeft;
												positionTop := nextTop;
											end;

										if positionTop < gameGlobals.borderTop then
											begin
												positionTop := gameGlobals.borderTop;
												direction := Stay;
												frameNo := 0;
											end;
									end;

								Right: 
									begin
										incrementX := gameGlobals.incrementSize;
										incrementY := 0;
										nextLeft := positionLeft + incrementX;
										nextTop := positionTop + incrementY;
										SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
										InsetRect(nextRect, insetPadding, insetPadding);

										wallCollisionFlag := WallCollisionTest(nextRect);
										if wallCollisionFlag then
											direction := Stay
										else
											begin
												positionLeft := nextLeft;
												positionTop := nextTop;
											end;

										if positionLeft > gameGlobals.borderRight then
											begin
												positionLeft := gameGlobals.borderRight;
												direction := Stay;
												frameNo := 0;
											end;
									end;

								Down: 
									begin
										incrementX := 0;
										incrementY := gameGlobals.incrementSize;
										nextLeft := positionLeft + incrementX;
										nextTop := positionTop + incrementY;
										SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
										InsetRect(nextRect, insetPadding, insetPadding);

										wallCollisionFlag := WallCollisionTest(nextRect);
										if wallCollisionFlag then
											direction := Stay
										else
											begin
												positionLeft := nextLeft;
												positionTop := nextTop;
											end;

										if positionTop > gameGlobals.borderBottom then
											begin
												positionTop := gameGlobals.borderBottom;
												direction := Stay;
												frameNo := 0;
											end;
									end;

								Stay: 
									begin
										incrementX := 0;
										incrementY := 0;
										positionLeft := positionLeft + incrementX;
										positionTop := positionTop + incrementY;
									end;

								otherwise
									AlertWithMessage('SetNewPosition case direction error, invalid case');
							end; {case direction of}

							SetRect(pictRect, positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
						end; {with pacmanInfo do}
				end; { PacMan }
		end; { case who of }


{ test for next desired direction, based on key presses }
		with pacmanInfo do
			begin
				case gameGlobals.desiredDirection of
					Left: 
						begin
							incrementX := gameGlobals.incrementSize * (-1);
							incrementY := 0;
							nextLeft := positionLeft + incrementX;
							nextTop := positionTop + incrementY;
							SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
							InsetRect(nextRect, insetPadding, insetPadding);

							wallCollisionFlag := WallCollisionTest(nextRect);
							if not wallCollisionFlag then
								direction := Left;
						end;

					Up: 
						begin
							incrementX := 0;
							incrementY := gameGlobals.incrementSize * (-1);
							nextLeft := positionLeft + incrementX;
							nextTop := positionTop + incrementY;
							SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
							InsetRect(nextRect, insetPadding, insetPadding);

							wallCollisionFlag := WallCollisionTest(nextRect);
							if not wallCollisionFlag then
								direction := Up;
						end;

					Right: 
						begin
							incrementX := gameGlobals.incrementSize;
							incrementY := 0;
							nextLeft := positionLeft + incrementX;
							nextTop := positionTop + incrementY;
							SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
							InsetRect(nextRect, insetPadding, insetPadding);

							wallCollisionFlag := WallCollisionTest(nextRect);
							if not wallCollisionFlag then
								direction := Right;
						end;

					Down: 
						begin
							incrementX := 0;
							incrementY := gameGlobals.incrementSize;
							nextLeft := positionLeft + incrementX;
							nextTop := positionTop + incrementY;
							SetRect(nextRect, nextLeft, nextTop, nextLeft + pictSize, nextTop + pictSize);
							InsetRect(nextRect, insetPadding, insetPadding);

							wallCollisionFlag := WallCollisionTest(nextRect);
							if not wallCollisionFlag then
								direction := Down;
						end;

					Stay: 
						;

					otherwise
						AlertWithMessage('SetNewPosition case direction error, invalid case');
				end; {case gameGlobals.desiredDirection of}
			end; { with pacmanInfo do }



	end;

{----------------------------}

	procedure ControlsHandler;

	begin
		case appGlobals.lastKeyPressed of
			'w', 'W', 'i', 'I':
{    pacmanInfo.direction := Up;}
				gameGlobals.desiredDirection := Up;
			'a', 'A', 'j', 'J': 
				gameGlobals.desiredDirection := Left;
			's', 'S', 'k', 'K': 
				gameGlobals.desiredDirection := Down;
			'd', 'D', 'l', 'L': 
				gameGlobals.desiredDirection := Right;
			otherwise
				;
		end;
	end;

{----------------------------}

	procedure PelletCollisionHandler;

		var
			xIndex, yIndex, bitSmallerRect: integer;
			collisionFlag, testResult: boolean;
			originalPacRect, pacmanRect, pelletRect, wallRect, intersectionRect: Rect;

	begin
		pacmanRect := pacmanInfo.pictRect;
		originalPacRect := pacmanRect;

{ create a testingRect a bit smaller so that pellets too-close to edges }
{ don 't cause multiple test-positives in the same frame }
		bitSmallerRect := round(pacmanInfo.pictSize / 6);
		InsetRect(pacmanRect, bitSmallerRect, bitSmallerRect);

		with pelletInfo do
			begin
				collisionFlag := false;

{ currently test against all 868 matrix slots }
{ probably a way to make this a bit more efficient based on current location }
				for yIndex := 0 to 30 do
					for xIndex := 0 to 27 do
						begin
							pelletRect := pelletInfo.rectArray[xIndex, yIndex];
							testResult := SectRect(pacmanRect, pelletRect, intersectionRect);
							if testResult then
								begin
									collisionFlag := true;
									gameGlobals.pelletsConsumed := gameGlobals.pelletsConsumed + 1;
{ 10 points per pellet }
									gameGlobals.scorePlayer1 := gameGlobals.scorePlayer1 + 10;
{ erase pellet }
{ NOTE: I notice that using black as fill causes slight discolorization }
{           due to backdrop image not being solid black}
									FillRect(pelletRect, black);
									SetRect(pelletInfo.rectArray[xIndex, yIndex], 0, 0, 0, 0);
								end;
						end;
			end; {with pelletInfo do}
	end;

{----------------------------}

	function WallCollisionTest (characterRect: Rect): boolean;

		var
			xIndex, yIndex: integer;
			testResult: boolean;
			wallRect, intersectionRect: Rect;

	begin
		WallCollisionTest := false;

{ currently test against all 868 matrix slots }
{ probably a way to make this a bit more efficient based on current location }
		for yIndex := 0 to 30 do
			for xIndex := 0 to 27 do
				begin
					wallRect := wallInfo.rectArray[xIndex, yIndex];
					testResult := SectRect(characterRect, wallRect, intersectionRect);
					if testResult then
						WallCollisionTest := true;
				end;
	end;

{----------------------------}

end.