unit Graphics;

interface

	uses
		QDOffscreen, Globals, Dialogs, Game;

{----------------------------}

	procedure InitGraphics;
	procedure DrawScreen (entireFlag: boolean);

{----------------------------}

implementation

{----------------------------}

	procedure BuildOffWorld (sizeOfDoc: Rect);

		var
			oldPerm: boolean;
			currDevice: GDHandle;
			currPort: GWorldPtr;
			erry: OSErr;

	begin
		GetGWorld(currPort, currDevice);
		erry := NewGWorld(gameWindow.offGWorld, 16, sizeOfDoc, nil, nil, GWorldFlags(0));

		SetGWorld(gameWindow.offGWorld, nil);
		if (LockPixels(gameWindow.offGWorld^.portPixMap)) then
			begin
				EraseRect(gameWindow.offGWorld^.portRect);
				UnlockPixels(gameWindow.offGWorld^.portPixMap);
			end;

		SetGWorld(currPort, currDevice);
	end;

{----------------------------}

	procedure InitPict_Backdrop;

		var
			resourceRect: Rect;

	begin
		with backgroundPict do
			begin
				if gColorQdAvailable then
					if gameMenus.windowSelection = Small then
						pictH := GetPicture(BACKDROP_ID_BW_SML)
					else if gameMenus.windowSelection = Medium then
						pictH := GetPicture(BACKDROP_ID_C_MED)
					else if gameMenus.windowSelection = Large then
						pictH := GetPicture(BACKDROP_ID_C_LRG)
					else
						exit(InitPict_Backdrop)
				else
					pictH := GetPicture(BACKDROP_ID_BW_SML);

				resourceRect := backgroundPict.pictH^^.picFrame;
				pictWidth := resourceRect.right - resourceRect.left;
				pictHeight := resourceRect.bottom - resourceRect.top;
				pictLeft := round((gameWindow.width - pictWidth) / 2);
				pictTop := round((gameWindow.height - pictHeight) / 2);
				SetRect(pictRect, pictLeft, pictTop, pictLeft + pictWidth, pictTop + pictHeight);
			end;
	end;

{----------------------------}

	procedure InitPict_Pac;

		var
			resourceRect: Rect;
			index: integer;

	begin
		case gameMenus.windowSelection of
			Small: 
				begin
					for index := 1 to 9 do
						with pacmanInfo.pictArray[index] do
							pictH := GetPicture(PACMAN_1STPICT_SML - 1 + index);
					with pacmanInfo do
						begin
							resourceRect := pictArray[2].pictH^^.picFrame;
							pictSize := resourceRect.right - resourceRect.left;		{ 16 }
						end;
				end;

			Medium: 
				begin
					for index := 1 to 9 do
						with pacmanInfo.pictArray[index] do
							pictH := GetPicture(PACMAN_1STPICT_MED - 1 + index);
					with pacmanInfo do
						begin
							resourceRect := pictArray[2].pictH^^.picFrame;
							pictSize := resourceRect.right - resourceRect.left;		{ 26 }
						end;
				end;

			Large: 
				begin
					for index := 1 to 9 do
						with pacmanInfo.pictArray[index] do
							pictH := GetPicture(PACMAN_1STPICT_LRG - 1 + index);
					with pacmanInfo do
						begin
							resourceRect := pictArray[2].pictH^^.picFrame;
							pictSize := resourceRect.right - resourceRect.left;		{ 45 }
							pictSize := 44;		{ purposely small; down from 45 }
						end;
				end;

			otherwise
				AlertWithMessage('InitPict_Pac case gameMenus.windowSelection error, invalid case');

		end;
	end;

{----------------------------}

	procedure InitPict_Pellet;

		var
			resourceRect: Rect;
			bitBoolean: array[0..31] of boolean;
			matrixPtr: longIntPtr;
			index: integer;

	begin
		with pelletInfo do
			begin
				matrix[0] := P_MATRIX_00;
				matrix[1] := P_MATRIX_01;
				matrix[2] := P_MATRIX_02;
				matrix[3] := P_MATRIX_03;
				matrix[4] := P_MATRIX_04;
				matrix[5] := P_MATRIX_05;
				matrix[6] := P_MATRIX_06;
				matrix[7] := P_MATRIX_07;
				matrix[8] := P_MATRIX_08;
				matrix[9] := P_MATRIX_09;
				matrix[10] := P_MATRIX_10;
				matrix[11] := P_MATRIX_11;
				matrix[12] := P_MATRIX_12;
				matrix[13] := P_MATRIX_13;
				matrix[14] := P_MATRIX_14;
				matrix[15] := P_MATRIX_15;
				matrix[16] := P_MATRIX_16;
				matrix[17] := P_MATRIX_17;
				matrix[18] := P_MATRIX_18;
				matrix[19] := P_MATRIX_19;
				matrix[20] := P_MATRIX_20;
				matrix[21] := P_MATRIX_21;
				matrix[22] := P_MATRIX_22;
				matrix[23] := P_MATRIX_23;
				matrix[24] := P_MATRIX_24;
				matrix[25] := P_MATRIX_25;
				matrix[26] := P_MATRIX_26;
				matrix[27] := P_MATRIX_27;
				matrix[28] := P_MATRIX_28;
				matrix[29] := P_MATRIX_29;
				matrix[30] := P_MATRIX_30;

{ to view in Lightsbug }
				matrixPtr := @matrix[0];
				for index := 0 to 31 do
					bitBoolean[index] := BitTst(Ptr(matrixPtr), 31 - index);	{ reverse order }
			end;

		case gameMenus.windowSelection of
			Small: 
				with pelletInfo do
					begin
						pictImage.pictH := GetPicture(PELLET_MED);  { couldn't get small pellet to draw }
						resourceRect := pictImage.pictH^^.picFrame;
						pictSize := resourceRect.right - resourceRect.left;
					end;

			Medium: 
				with pelletInfo do
					begin
						pictImage.pictH := GetPicture(PELLET_MED);
						resourceRect := pictImage.pictH^^.picFrame;
						pictSize := resourceRect.right - resourceRect.left;
					end;

			Large: 
				with pelletInfo do
					begin
						pictImage.pictH := GetPicture(PELLET_LRG);
						resourceRect := pictImage.pictH^^.picFrame;
						pictSize := resourceRect.right - resourceRect.left;
					end;

			otherwise
				AlertWithMessage('InitPict_Pellet case gameMenus.windowSelection error, invalid case');

		end;
	end;

{----------------------------}

	procedure InitPict_Digit;

		var
			index: integer;
			resourceRect: Rect;

	begin
		with digitInfo do
			begin
				case gameMenus.windowSelection of
					Small: 
						for index := 0 to 9 do
							begin
								if gColorQdAvailable then
									if gameMenus.windowSelection = Small then
										pictArray[index].pictH := GetPicture(DIGIT_0_SML + index)
									else if gameMenus.windowSelection = Medium then
										pictArray[index].pictH := GetPicture(DIGIT_0_SML + index)
									else if gameMenus.windowSelection = Large then
										pictArray[index].pictH := GetPicture(DIGIT_0_SML + index)
									else
										exit(InitPict_Digit)
								else
									pictArray[index].pictH := GetPicture(DIGIT_0_SML + index);

								pictRect := pictArray[index].pictH^^.picFrame;
								pictWidth := pictRect.right - pictRect.left;
								pictHeight := pictRect.bottom - pictRect.top;
							end;

					Medium: 
						for index := 0 to 9 do
							begin
								if gColorQdAvailable then
									if gameMenus.windowSelection = Small then
										pictArray[index].pictH := GetPicture(DIGIT_0_MED + index)
									else if gameMenus.windowSelection = Medium then
										pictArray[index].pictH := GetPicture(DIGIT_0_MED + index)
									else if gameMenus.windowSelection = Large then
										pictArray[index].pictH := GetPicture(DIGIT_0_MED + index)
									else
										exit(InitPict_Digit)
								else
									pictArray[index].pictH := GetPicture(DIGIT_0_MED + index);

								pictRect := pictArray[index].pictH^^.picFrame;
								pictWidth := pictRect.right - pictRect.left;
								pictHeight := pictRect.bottom - pictRect.top;
							end;

					Large: 
						for index := 0 to 9 do
							begin
								if gColorQdAvailable then
									if gameMenus.windowSelection = Small then
										pictArray[index].pictH := GetPicture(DIGIT_0_LRG + index)
									else if gameMenus.windowSelection = Medium then
										pictArray[index].pictH := GetPicture(DIGIT_0_LRG + index)
									else if gameMenus.windowSelection = Large then
										pictArray[index].pictH := GetPicture(DIGIT_0_LRG + index)
									else
										exit(InitPict_Digit)
								else
									pictArray[index].pictH := GetPicture(DIGIT_0_LRG + index);

								pictRect := pictArray[index].pictH^^.picFrame;
								pictWidth := pictRect.right - pictRect.left;
								pictHeight := pictRect.bottom - pictRect.top;
							end;

					otherwise
						AlertWithMessage('InitPict_Digit case gameMenus.windowSelection error, invalid case');

				end; { case gameMenus.windowSelection of }
			end; { with digitInfo do }
	end;

{----------------------------}

	procedure InitWalls;

		var
			resourceRect: Rect;
			index: integer;

	begin
		with wallInfo do
			begin
				matrix[0] := W_MATRIX_00;
				matrix[1] := W_MATRIX_01;
				matrix[2] := W_MATRIX_02;
				matrix[3] := W_MATRIX_03;
				matrix[4] := W_MATRIX_04;
				matrix[5] := W_MATRIX_05;
				matrix[6] := W_MATRIX_06;
				matrix[7] := W_MATRIX_07;
				matrix[8] := W_MATRIX_08;
				matrix[9] := W_MATRIX_09;
				matrix[10] := W_MATRIX_10;
				matrix[11] := W_MATRIX_11;
				matrix[12] := W_MATRIX_12;
				matrix[13] := W_MATRIX_13;
				matrix[14] := W_MATRIX_14;
				matrix[15] := W_MATRIX_15;
				matrix[16] := W_MATRIX_16;
				matrix[17] := W_MATRIX_17;
				matrix[18] := W_MATRIX_18;
				matrix[19] := W_MATRIX_19;
				matrix[20] := W_MATRIX_20;
				matrix[21] := W_MATRIX_21;
				matrix[22] := W_MATRIX_22;
				matrix[23] := W_MATRIX_23;
				matrix[24] := W_MATRIX_24;
				matrix[25] := W_MATRIX_25;
				matrix[26] := W_MATRIX_26;
				matrix[27] := W_MATRIX_27;
				matrix[28] := W_MATRIX_28;
			end;
	end;

{----------------------------}

	procedure InitGraphics;

	begin
		BuildOffWorld(gameWindow.windowPtr^.portRect);
		InitPict_Backdrop;
		InitPict_Pac;
		InitPict_Pellet;
		InitPict_Digit;
		InitWalls;
	end;

{----------------------------}

	procedure DrawScore;

		var
			score, scoreDigitCount, digit5, digit4, digit3, digit2, digit1: longint;
			firstDigitLeft, firstDigitTop: integer;
			scoreStr: Str255;
			eraseRect: Rect;

	begin
		score := gameGlobals.scorePlayer1;
		digit1 := 0;
		digit2 := 0;
		NumToString(score, scoreStr);

		case gameMenus.windowSelection of
			Small: 
				begin
					firstDigitLeft := DIGIT_1ST_LT_SML;
					firstDigitTop := DIGIT_1ST_TOP_SML;
				end;
			Medium: 
				begin
					firstDigitLeft := DIGIT_1ST_LT_MED;
					firstDigitTop := DIGIT_1ST_TOP_MED;
				end;
			Large: 
				begin
					firstDigitLeft := DIGIT_1ST_LT_LRG;
					firstDigitTop := DIGIT_1ST_TOP_LRG;
				end;

			otherwise
				AlertWithMessage('DrawScore case gameMenus.windowSelection error, invalid case');
		end;

		if score < 10 then
			begin
				scoreDigitCount := 1;
				StringToNum(scoreStr[1], digit1);
{ 2-bit PICTs appear to not just work as srcCopy, but seem to need a white fill to be drawn onto }
				SetRect(eraseRect, firstDigitLeft + (digitInfo.pictWidth * 4), firstDigitTop, firstDigitLeft + (digitInfo.pictWidth * 4) + digitInfo.pictWidth, firstDigitTop + digitInfo.pictHeight);
				FillRect(eraseRect, white);
			end
		else if score < 100 then
			begin
				scoreDigitCount := 2;
				StringToNum(scoreStr[1], digit2);
				StringToNum(scoreStr[2], digit1);
{ 2-bit PICTs appear to not just work as srcCopy, but seem to need a white fill to be drawn onto }
				SetRect(eraseRect, firstDigitLeft + (digitInfo.pictWidth * 3), firstDigitTop, firstDigitLeft + (digitInfo.pictWidth * 4) + digitInfo.pictWidth, firstDigitTop + digitInfo.pictHeight);
				FillRect(eraseRect, white);
			end
		else if score < 1000 then
			begin
				scoreDigitCount := 3;
				StringToNum(scoreStr[1], digit3);
				StringToNum(scoreStr[2], digit2);
				StringToNum(scoreStr[3], digit1);
{ 2-bit PICTs appear to not just work as srcCopy, but seem to need a white fill to be drawn onto }
				SetRect(eraseRect, firstDigitLeft + (digitInfo.pictWidth * 2), firstDigitTop, firstDigitLeft + (digitInfo.pictWidth * 4) + digitInfo.pictWidth, firstDigitTop + digitInfo.pictHeight);
				FillRect(eraseRect, white);
			end
		else if score < 10000 then
			begin
				scoreDigitCount := 4;
				StringToNum(scoreStr[1], digit4);
				StringToNum(scoreStr[2], digit3);
				StringToNum(scoreStr[3], digit2);
				StringToNum(scoreStr[4], digit1);
{ 2-bit PICTs appear to not just work as srcCopy, but seem to need a white fill to be drawn onto }
				SetRect(eraseRect, firstDigitLeft + (digitInfo.pictWidth * 1), firstDigitTop, firstDigitLeft + (digitInfo.pictWidth * 4) + digitInfo.pictWidth, firstDigitTop + digitInfo.pictHeight);
				FillRect(eraseRect, white);
			end
		else if score < 100000 then
			begin
				scoreDigitCount := 5;
				StringToNum(scoreStr[1], digit5);
				StringToNum(scoreStr[2], digit4);
				StringToNum(scoreStr[3], digit3);
				StringToNum(scoreStr[4], digit2);
				StringToNum(scoreStr[5], digit1);
{ 2-bit PICTs appear to not just work as srcCopy, but seem to need a white fill to be drawn onto }
				SetRect(eraseRect, firstDigitLeft + (digitInfo.pictWidth * 0), firstDigitTop, firstDigitLeft + (digitInfo.pictWidth * 4) + digitInfo.pictWidth, firstDigitTop + digitInfo.pictHeight);
				FillRect(eraseRect, white);
			end;



		with digitInfo do
			begin
{ ten thousands column }
				positionLeft := firstDigitLeft + (pictWidth * 0);
				positionTop := firstDigitTop;
				SetRect(pictRect, positionLeft, positionTop, positionLeft + pictWidth, positionTop + pictHeight);
				if scoreDigitCount = 5 then
					DrawPicture(pictArray[digit5].pictH, pictRect);

{ thousands column }
				positionLeft := firstDigitLeft + (pictWidth * 1);
				positionTop := firstDigitTop;
				SetRect(pictRect, positionLeft, positionTop, positionLeft + pictWidth, positionTop + pictHeight);
				if scoreDigitCount >= 4 then
					DrawPicture(pictArray[digit4].pictH, pictRect);

{ hundreds column }
				positionLeft := firstDigitLeft + (pictWidth * 2);
				positionTop := firstDigitTop;
				SetRect(pictRect, positionLeft, positionTop, positionLeft + pictWidth, positionTop + pictHeight);
				if scoreDigitCount >= 3 then
					DrawPicture(pictArray[digit3].pictH, pictRect);

{ tens column }
				positionLeft := firstDigitLeft + (pictWidth * 3);
				positionTop := firstDigitTop;
				SetRect(pictRect, positionLeft, positionTop, positionLeft + pictWidth, positionTop + pictHeight);
				if (scoreDigitCount >= 2) or (score = 0) then
					DrawPicture(pictArray[digit2].pictH, pictRect);

{ ones column }
				positionLeft := firstDigitLeft + (pictWidth * 4);
				positionTop := firstDigitTop;
				SetRect(pictRect, positionLeft, positionTop, positionLeft + pictWidth, positionTop + pictHeight);
				DrawPicture(pictArray[digit1].pictH, pictRect);
			end;
	end;

{----------------------------}

	procedure DrawStats;

		var
			ticksAsString, pelletsAsString: Str255;
			pacLeftAsString, pacTopAsString: Str255;
			textRect: Rect;

	begin
{ tick data }
		SetRect(textRect, STAT_TICK_LEFT, STAT_TICK_TOP, STAT_TICK_LEFT + 100, STAT_TICK_TOP + 10);
		FillRect(textRect, black);
		PenPat(white);
		MoveTo(STAT_TICK_LEFT, STAT_TICK_TOP + 10);
		TextMode(srcXor);
		NumToString(appGlobals.avgDrawCycles, ticksAsString);
		DrawString(concat('ticks/draw:', ticksAsString));

{ pellet data }
		SetRect(textRect, STAT_PELLET_LEFT, STAT_PELLET_TOP, STAT_PELLET_LEFT + 110, STAT_PELLET_TOP + 12);
		FillRect(textRect, black);
		PenPat(white);
		MoveTo(STAT_PELLET_LEFT, STAT_PELLET_TOP + 10);
		TextMode(srcXor);
		NumToString(gameGlobals.pelletsConsumed, pelletsAsString);
		DrawString(concat('pellets:', pelletsAsString));

{ pacman data }
		SetRect(textRect, STAT_LOC_LEFT, STAT_LOC_TOP, STAT_LOC_LEFT + 110, STAT_LOC_TOP + 12);
		FillRect(textRect, black);
		PenPat(white);
		MoveTo(STAT_LOC_LEFT, STAT_LOC_TOP + 10);
		TextMode(srcXor);
		NumToString(pacmanInfo.positionLeft, pacLeftAsString);
		NumToString(pacmanInfo.positionTop, pacTopAsString);
		DrawString(concat('pac:', pacLeftAsString, ', ', pacTopAsString));
	end;

{----------------------------}

	procedure DrawPac (offscreenPixMap: PixMap);		{ w/ PelletCollisionHandling }

		var
			offscreenRect: Rect;
			offscreenBitMapPtr: BitMapPtr;
			pacPreviousLeft, pacPreviousTop: integer;

	begin
{ create an offscreen spot to temorarily copy/render the previous pixels, of the new location }
		offscreenBitMapPtr := BitMapPtr(@offscreenPixMap);

		with pacmanInfo do
			begin
				pacPreviousLeft := PREV_PAC_LEFT;
				pacPreviousTop := PREV_PAC_TOP;

{ restore the previous pixels (erase existing pacman) }
				SetRect(offscreenRect, pacPreviousLeft, pacPreviousTop, pacPreviousLeft + pacmanInfo.pictSize, pacPreviousTop + pacmanInfo.pictSize);
				CopyBits(offscreenBitMapPtr^, offscreenBitMapPtr^, offscreenRect, pacmanInfo.pictRect, srcCopy, nil);

{ each animation frame needs a different image - currently 3 frames for pacman }
				frameNo := frameNo + 1;
				if frameNo > 2 then
					frameNo := 0;

{ handle collision testing here so that eaten pellets can disappear before storing current frame }
				if not gameMenus.disableCollisionsFlag then
					PelletCollisionHandler;

{ determine new position for pacman }
				SetNewPosition(PacMan);

{ capture existing pixels of new location }
				SetRect(offscreenRect, pacPreviousLeft, pacPreviousTop, pacPreviousLeft + pacmanInfo.pictSize, pacPreviousTop + pacmanInfo.pictSize);
				CopyBits(offscreenBitMapPtr^, offscreenBitMapPtr^, pacmanInfo.pictRect, offscreenRect, srcCopy, nil);

{ draw a white rect around the working area, just for developer's visual reference }
				PenPat(white);
				InsetRect(offscreenRect, -1, -1);
				FrameRect(offscreenrect);

{ draw pacman image at the new location, based on frameNo }
				case direction of
					Left: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[2].pictH, pictRect)
						else
							DrawPicture(pictArray[3].pictH, pictRect);

					Up: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[8].pictH, pictRect)
						else
							DrawPicture(pictArray[9].pictH, pictRect);

					Right: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[4].pictH, pictRect)
						else
							DrawPicture(pictArray[5].pictH, pictRect);

					Down: 
						if frameNo = 0 then
							DrawPicture(pictArray[1].pictH, pictRect)
						else if frameNo = 1 then
							DrawPicture(pictArray[6].pictH, pictRect)
						else
							DrawPicture(pictArray[7].pictH, pictRect);

					Stay: 
						DrawPicture(pictArray[1].pictH, pictRect);

					otherwise
						AlertWithMessage('DrawPacWithCollisionHandling case direction error, invalid case');
				end;
			end;
	end;

{----------------------------}

	procedure DrawPellets;

		var
			positionLeft, positionTop, index, xIndex, yIndex: integer;
			firstPelletLeft, firstPelletTop: integer;
			firstPelletSpacing: real;
			pictRect: Rect;
			matrixPtr: longintPtr;

	begin
		case gameMenus.windowSelection of
			Small: 
				begin
					with pelletInfo do
						begin
							firstPelletLeft := PLT_1ST_LT_SML;
							firstPelletTop := PLT_1ST_TOP_SML;
							firstPelletSpacing := PLT_1ST_SPACE_SML;

							for yIndex := 0 to 30 do
								for xIndex := 0 to 27 do
									begin
										positionLeft := firstPelletLeft + round(xIndex * firstPelletSpacing);
										positionTop := firstPelletTop + round(yIndex * firstPelletSpacing);
										matrixPtr := @matrix[yIndex];

										if gameMenus.viewPelletMatrixFlag then
											begin
												SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
												DrawPicture(pictImage.pictH, rectArray[xIndex, yIndex]);
											end
										else if BitTst(Ptr(matrixPtr), 31 - xIndex) then
											begin
												SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
												DrawPicture(pictImage.pictH, rectArray[xIndex, yIndex]);
											end;
									end;
						end;
				end;

			Medium: 
				begin
					with pelletInfo do
						begin
							firstPelletLeft := PLT_1ST_LT_MED;
							firstPelletTop := PLT_1ST_TOP_MED;
							firstPelletSpacing := PLT_1ST_SPACE_MED;

							for yIndex := 0 to 30 do
								for xIndex := 0 to 27 do
									begin
										positionLeft := firstPelletLeft + round(xIndex * firstPelletSpacing);
										positionTop := firstPelletTop + round(yIndex * firstPelletSpacing);
										matrixPtr := @matrix[yIndex];

										if gameMenus.viewPelletMatrixFlag then
											begin
												SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
												DrawPicture(pictImage.pictH, rectArray[xIndex, yIndex]);
											end
										else if BitTst(Ptr(matrixPtr), 31 - xIndex) then
											begin
												SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
												DrawPicture(pictImage.pictH, rectArray[xIndex, yIndex]);
											end;
									end;
						end;
				end;

			Large: 
				begin
					with pelletInfo do
						begin
							firstPelletLeft := PLT_1ST_LT_LRG;
							firstPelletTop := PLT_1ST_TOP_LRG;
							firstPelletSpacing := PLT_1ST_SPACE_LRG;

							for yIndex := 0 to 30 do
								for xIndex := 0 to 27 do
									begin
										positionLeft := firstPelletLeft + round(xIndex * firstPelletSpacing);
										positionTop := firstPelletTop + round(yIndex * firstPelletSpacing);
										matrixPtr := @matrix[yIndex];

										if gameMenus.viewPelletMatrixFlag then
											begin
												SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
												DrawPicture(pictImage.pictH, rectArray[xIndex, yIndex]);
											end
										else if BitTst(Ptr(matrixPtr), 31 - xIndex) then
											begin
												SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + pictSize, positionTop + pictSize);
												DrawPicture(pictImage.pictH, rectArray[xIndex, yIndex]);
											end;
									end;

						end;
				end;

			otherwise
				AlertWithMessage('DrawPellets case gameMenus.windowSelection error, invalid case');

		end;
	end;

{----------------------------}

	procedure CreateWalls;

		var
			positionLeft, positionTop, index, xIndex, yIndex: integer;
			firstPelletLeft, firstPelletTop, wallPadding: integer;
			firstPelletSpacing: real;
			pictRect: Rect;
			matrixPtr: longintPtr;

	begin
		with wallInfo do
			begin
				case gameMenus.windowSelection of
					Small: 
						begin
							firstPelletLeft := PLT_1ST_LT_SML;
							firstPelletTop := PLT_1ST_TOP_SML;
							firstPelletSpacing := PLT_1ST_SPACE_SML;
							wallPadding := 2;
						end;
					Medium: 
						begin
							firstPelletLeft := PLT_1ST_LT_MED;
							firstPelletTop := PLT_1ST_TOP_MED;
							firstPelletSpacing := PLT_1ST_SPACE_MED;
							wallPadding := 2;
						end;
					Large: 
						begin
							firstPelletLeft := PLT_1ST_LT_LRG;
							firstPelletTop := PLT_1ST_TOP_LRG;
							firstPelletSpacing := PLT_1ST_SPACE_LRG;
							wallPadding := 5;
						end;

					otherwise
						AlertWithMessage('CreateWalls case gameMenus.windowSelection error, invalid case');
				end;

				for yIndex := 0 to 28 do
					for xIndex := 0 to 27 do
						begin
							positionLeft := firstPelletLeft + round(xIndex * firstPelletSpacing) + wallPadding;
							positionTop := firstPelletTop + round(yIndex * firstPelletSpacing) + wallPadding;
							matrixPtr := @matrix[yIndex];

							if BitTst(Ptr(matrixPtr), 31 - xIndex) then
								begin
									SetRect(rectArray[xIndex, yIndex], positionLeft, positionTop, positionLeft + round(firstPelletSpacing), positionTop + round(firstPelletSpacing));

									if gameMenus.viewWallMatrixFlag then
										FillRect(rectArray[xIndex, yIndex], ltGray);
								end;
						end;
			end;
	end;

{----------------------------}

	procedure DrawBackdrop;

	begin
		with backgroundPict do
			begin
				FillRect(pictRect, white);
				DrawPicture(pictH, pictRect);
			end;
	end;

{----------------------------}

	procedure DrawScreen (entireFlag: boolean);

		var
			offscreenPixMap, windowPixMap: PixMap;
			windowBitMap: BitMap;
			screenRect, gameRect, statsRect: Rect;
			currPort: GWorldPtr;
			currDevice: GDHandle;

	begin
		GetGWorld(currPort, currDevice);
		SetGWorld(gameWindow.offGWorld, nil);
		offscreenPixMap := GetGWorldPixMap(gameWindow.offGWorld)^^;
		if (LockPixels(GetGWorldPixMap(gameWindow.offGWorld))) then
			begin

				if entireFlag then
					begin
{ draw all game elements onto offscreen PixMap }
{ background color }
						FillRect(gameWindow.offGWorld^.portRect, black);
						DrawBackdrop;
						DrawPellets;
						CreateWalls;
						DrawPac(offscreenPixMap);		{ w/ PelletCollisionHandling }
						if gameMenus.viewStatsFlag then
							DrawStats;
						DrawScore;
					end
				else
					begin
{ draw only applicable game elements onto offscreen PixMap }
						DrawPac(offscreenPixMap);		{ w/ PelletCollisionHandling }
						if gameMenus.viewStatsFlag then
							DrawStats;
						DrawScore;
					end;

				UnlockPixels(GetGWorldPixMap(gameWindow.offGWorld));
			end;

{ select gameWindow }
		SetGWorld(currPort, currDevice);
{ background color }
		if entireFlag then
			FillRect(gameWindow.windowPtr^.portRect, black);

{ offscreenPixMap fully drawn, now copy it to the gameWindow PixMap }
		screenRect := gameWindow.windowPtr^.portRect;
		gameRect := backgroundPict.pictRect;

		if gameMenus.windowSelection = Small then
			begin
				windowBitMap := gameWindow.windowPtr^.portBits;
				CopyBits(BitMapPtr(@offscreenPixMap)^, windowBitMap, screenRect, screenRect, srcCopy, nil);
			end

		else if gameMenus.windowSelection = Medium then
			begin
				windowPixMap := gameWindow.cWindowPtr^.portPixMap^^;
				CopyBits(BitMapPtr(@offscreenPixMap)^, BitMapPtr(@windowPixMap)^, screenRect, screenRect, srcCopy, nil)
			end

		else if gameMenus.windowSelection = Large then
			begin
				windowPixMap := gameWindow.cWindowPtr^.portPixMap^^;
				CopyBits(BitMapPtr(@offscreenPixMap)^, BitMapPtr(@windowPixMap)^, screenRect, screenRect, srcCopy, nil)
			end;

		appGlobals.lastDrawTick := TickCount;
		appGlobals.drawCycles := appGlobals.drawCycles + 1;
	end;

{----------------------------}

end.